import logging
from pathlib import Path
import click

logging.basicConfig(
    level = "DEBUG", format = "'%(asctime)s - %(name)s - %(levelname)s - %(message)s'"
)
logger = logging.getLogger(__name__)

@click.command()
@click.option(
    "--input", 
    "-i", 
    default = "./", 
    help="Path were to find the JSON files to be converted as CSV.", 
    type=str,
)
@click.option(
    "--output", 
    "-o", 
    default = "./", 
    help="Path were the converted files will be saved.", 
    type=str,
)
@click.option(
    "--prefix", 
    "-p",
    prompt = True,
    prompt_required = False, 
    default = "file", 
    help=(
        "Prefix used to prepend to the name of the converted file saved on disk."
        "The suffix will be a number starting from 0. ge: file_0.json"), 
)

def converterj(input: str = "./", output: str = "./", delimiter: str = "./", prefix: str = None):
    input_path = Path(input)
    output_path = Path(output)
    logger.info('Input Path: %s', input_path)
    logger.info('Output Path: %s', output_path)

    for p in [input_path, output_path]:
        if not (p.is_file() or p.is_dir()):
            raise TypeError('Not a valid path or file name.')

    data = read_json_file(source = input_path)
    save_to_csv_files(lists = data, output_path = output_path, prefix = prefix)

def raw_read_json(source: Path):
    """Faz a leitura de um arquivo JSON ou pasta contendo vários arquivos JSON"""
    with open(source) as file:
        data = file.readlines()
    parsed_data = [(obj.rstrip()).lstrip()[:-1] for obj in data[1:-1]]

    return parsed_data

def read_json_file(source: Path):
    """Load a single csv file or all files withing a directory.
    
    Args: 
        source (Path): Path for a single file or directory with files.
        delimiter(str, optional): Separator for columns in the csv's. Defaults to ','.
    
    Returns:
        tuple: All dataframes loades from the given source path.
    """
    if source.is_file():
        logger.info('Reading csv file %s', source)
        raw_read_json(source = source)

    logger.info('Reading all files within the directory: %s', source)
    data = list()
    for i in source.iterdir():
        data.append(raw_read_json(source = i))
    
    return(data)

def parse_json_to_csv(data: list):
    """Converte lista de dados para o formato de matriz"""

    # coloca { onde deveria ter
    data = ['{' if data[i] == '' else data[i] for i in range(len(data))]
    
    # retira aspas duplas 
    data = [string.replace('"', '') for string in data]
    
    # coloca ; no fim pra servir como delimitador depois 
    data = ["{}{}".format(i, '\t') for i in data]
    
    # separa as chaves e valores 
    keys_strings = []
    newer_strings = []

    for i in range(len(data)):
        partitions = data[i].partition(': ')
    
        new_key = partitions[0]
        new_string = partitions[2]
    
        newer_strings.append(new_string)
        keys_strings.append(new_key)
        
    # coloca ; nas chaves tb
    keys_strings = ["{}{}".format(i, '\t') for i in keys_strings]
    
    # Pega o nome das colunas repetidos e deixa em um unico valor
    uniqueList = []

    for letter in keys_strings:
        if letter not in uniqueList:
            uniqueList.append(letter)
        
    # Faz uma lista enorme com o nome das colunas + valores
    final_list = uniqueList + newer_strings
    
    # Forma uma matriz numa estrutura de um banco de dados
    col = len(uniqueList)
    data = [final_list[i:i + col] for i in range(0, len(final_list), col)]
    
    # Retira a primeira coluna
    for x in data:
        del x[0]

    # Retira a última coluna
    for x in data:
        del x[len(data[0])-1]
        
    return data

def write_csv_data(data, output_path: Path):
    """Escreve uma matriz em formato dataframe em disco"""
    with output_path.open(mode = 'w') as testfile:
        for row in data:
            testfile.write(' '.join([str(a) for a in row]) + '\n')
         
def save_to_csv_files(lists: list, output_path: Path, prefix: str = None):
    """Save dataframes to disk.
    
    Args:
        lists (list): list with dataframes that will be converted.
        output_path (Path): Path where to save the json files;
        prefix (str, optional): Name to prepend to files. If nothing is given, it will use file_.
    """
    i = 0
    while i < len(lists):
        file_name = f'{prefix}_{i}.csv'
        output = output_path.joinpath(file_name)
        logger.info('Saving file: %s', output)
        write_csv_data(parse_json_to_csv(lists[i]), output_path = output)
        i += 1





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['luana_santos_csv_converter']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0', 'pandas>=1.3.5,<2.0.0']

entry_points = \
{'console_scripts': ['csv_converter = '
                     'luana_santos_csv_converter.converter:converter',
                     'json_converter = '
                     'luana_santos_csv_converter.converterj:converterj']}

setup_kwargs = {
    'name': 'luana-santos-csv-converter',
    'version': '0.1.0',
    'description': 'Convert csv to json and vice versa. For learning purposes.',
    'long_description': '# File Converter\n\nCSV to JSON and JSON to CSV converter.\n\n## Introduction\n\nLearning how to deploy a lib on PyPi at PUC using Poetry.\n\n### What this project can do\n\nRead a **csv** file or a **folder** and convert them to **JSON** and vice versa.\nThis project is a program running on terminal, preferably install with pipx:\n\n```bash\npipx install luana_santos_csv_converter\n```\n\nTo use, just type in:\n\n```bash\ncsv_converter --help\n```\n\nOr:\n\n```bash\njson_converter --help\n```',
    'author': 'Luana Santos',
    'author_email': 'luanasilvia.lss@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import logging
from pathlib import Path
import click

logging.basicConfig(
    level = "DEBUG", format = "'%(asctime)s - %(name)s - %(levelname)s - %(message)s'"
)
logger = logging.getLogger(__name__)

@click.command()
@click.option(
    "--input", 
    "-i", 
    default = "./", 
    help="Path were to find the CSV files to be converted as JSON.", 
    type=str,
)
@click.option(
    "--output", 
    "-o", 
    default = "./", 
    help="Path were the converted files will be saved.", 
    type=str,
)
@click.option(
    "--delimiter", 
    "-d", 
    default = ",", 
    help="Separator used to split the files.", 
    type=str,
)
@click.option(
    "--prefix", 
    "-p",
    prompt = True,
    prompt_required = False, 
    default = "file", 
    help=(
        "Prefix used to prepend to the name of the converted file saved on disk."
        "The suffix will be a number starting from 0. ge: file_0.json"), 
)

def converter(input: str = "./", output: str = "./", delimiter: str = "./", prefix: str = None):
    input_path = Path(input)
    output_path = Path(output)
    logger.info('Input Path: %s', input_path)
    logger.info('Output Path: %s', output_path)

    for p in [input_path, output_path]:
        if not (p.is_file() or p.is_dir()):
            raise TypeError('Not a valid path or file name.')

    data = read_csv_file(source = input_path, delimiter = delimiter)
    save_to_json_files(csvs = data, output_path = output_path, prefix = prefix)

def raw_read_csv(source: Path, delimiter: str = ","):
    """Load a single csv file withing a directory."""
    with source.open(mode = "r") as file:
        dt = file.readlines()

    parsed_data = [line.strip().split(delimiter) for line in dt]
    return parsed_data

def read_csv_file(source: Path, delimiter: str = ','):
    """Load a single csv file or all files withing a directory.
    
    Args: 
        source (Path): Path for a single file or directory with files.
        delimiter(str, optional): Separator for columns in the csv's. Defaults to ','.
    
    Returns:
        tuple: All dataframes loades from the given source path.
    """
    if source.is_file():
        logger.info('Reading csv file %s', source)
        raw_read_csv(source = source, delimiter = delimiter)

    logger.info('Reading all files within the directory: %s', source)
    data = list()
    for i in source.iterdir():
        data.append(raw_read_csv(source = i, delimiter = delimiter))
    
    return(data)

def parse_csv_to_json(data: list[list[str]]):
    """Converte lista de dados para o formato de dicionário"""
    column = data[0]
    lines = data[1:]
    result = [dict(zip(column, line)) for line in lines]
    return result

def write_line(line: tuple, io, append_comma: bool = True):
    """Escreve uma linha do dicionario no disco"""
    key, value = line
    if append_comma:
        io.write(f'\t\t"{key}": "{value}", \n' )
    else:
         io.write(f'\t\t"{key}": "{value}" \n' )

def write_dictionary(data: dict, io, append_comma: bool = True):
    """Escreve dicionario no disco"""
    io.write('\t{\n')
    items = tuple(data.items())
    for line in items[:-1]:
        write_line(line, io, True)
    write_line(items[-1], io, False)
    io.write('\t}')
    if append_comma:
        io.write(',\n')
    else:
        io.write('\n')

def write_json_data(data: list[dict[str, str]], output_path: Path):
    """Escreve uma lista de dicionários em formato json em disco"""
    with output_path.open(mode = 'w') as file:
        file.write('[\n')
        for d in data[:-1]:
            write_dictionary(d, file, append_comma = True)
        write_dictionary(data[-1], file, append_comma = False)
        file.write(']\n')
         
def save_to_json_files(csvs: list, output_path: Path, prefix: str = None):
    """Save dataframes to disk.
    
    Args:
        csvs (list): List with dataframes that will be converted.
        output_path (Path): Path where to save the json files;
        prefix (str, optional): Name to prepend to files. If nothing is given, it will use file_.
    """
    i = 0
    while i < len(csvs):
        file_name = f'{prefix}_{i}.json'
        output = output_path.joinpath(file_name)
        logger.info('Saving file: %s', output)
        write_json_data(parse_csv_to_json(csvs[i]), output_path = output)
        i += 1


